import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsGrossAndInvalidTraffic(AssetSchema):
    """
    The Brands Gross and Invalid Traffic report provides insights into the quality of traffic generated by Amazon Ads
    campaigns focused on brand promotion. It includes key metrics such as gross impressions, gross click-throughs,
    invalid impressions, invalid click-throughs, invalid impression rate, and invalid click-through rate.
    """

    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    date: datetime.date | None = Field(..., description="The date of the data entry")
    gross_click_throughs: float | None = Field(..., description="The number of gross click throughs")
    gross_impressions: float | None = Field(..., description="The number of gross impressions")
    impressions: float | None = Field(..., description="The number of impressions")
    invalid_click_through_rate: float | None = Field(..., description="The rate of invalid click throughs")
    invalid_click_throughs: float | None = Field(..., description="The number of invalid click throughs")
    invalid_impression_rate: float | None = Field(..., description="The rate of invalid impressions")
    invalid_impressions: float | None = Field(..., description="The number of invalid impressions")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class ProductsGrossAndInvalidTraffic(AssetSchema):
    """
    The Products Gross and Invalid Traffic report provides insights into the performance of campaigns by revealing the
    impact of gross and invalid traffic on advertising efforts. It includes key metrics such as clicks, impressions,
    gross click-throughs, and invalid click-throughs.
    """

    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    date: datetime.date | None = Field(..., description="The date of the data entry")
    gross_click_throughs: float | None = Field(..., description="The number of gross click throughs")
    gross_impressions: float | None = Field(..., description="The number of gross impressions")
    impressions: float | None = Field(..., description="The number of impressions")
    invalid_click_through_rate: float | None = Field(..., description="The rate of invalid click throughs")
    invalid_click_throughs: float | None = Field(..., description="The number of invalid click throughs")
    invalid_impression_rate: float | None = Field(..., description="The rate of invalid impressions")
    invalid_impressions: float | None = Field(..., description="The number of invalid impressions")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsTargeting(AssetSchema):
    """
    The Brands Targeting report provides insights into ad performance based on various targeting criteria.
    It includes key metrics such as clicks, impressions, purchases, and sales, along with dimensions like ad group,
    campaign, keyword, and targeting expression.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_keyword_status: str | None = Field(..., description="The status of the ad keyword")
    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of clicks for add to cart")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of clicks for branded searches")
    campaign_budget_amount: float | None = Field(..., description="The amount of campaign budget")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_type: str | None = Field(..., description="The type of cost")
    date: datetime.date | None = Field(..., description="The date of the event")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of clicks for detail page views")
    ecp_add_to_cart: float | None = Field(..., description="The eCP add to cart")
    impressions: float | None = Field(..., description="The number of impressions")
    keyword_bid: float | None = Field(..., description="The bid for the keyword")
    keyword_id: int | None = Field(..., description="The ID of the keyword")
    keyword_text: str | None = Field(..., description="The text of the keyword")
    keyword_type: str | None = Field(..., description="The type of keyword")
    match_type: str | None = Field(..., description="The type of match")
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand detail page views"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(
        ..., description="The ECP detail page views for new-to-brand"
    )
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand purchases"
    )
    new_to_brand_purchases_percentage: float | None = Field(..., description="The percentage of new-to-brand purchases")
    new_to_brand_purchases_rate: float | None = Field(..., description="The rate of new-to-brand purchases")
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of clicks for new-to-brand sales")
    new_to_brand_sales_percentage: float | None = Field(..., description="The percentage of new-to-brand sales")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new-to-brand units sold")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand units sold"
    )
    new_to_brand_units_sold_percentage: float | None = Field(
        ..., description="The percentage of new-to-brand units sold"
    )
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of clicks for purchases")
    purchases_promoted: float | None = Field(..., description="The number of promoted purchases")
    sales: float | None = Field(..., description="The number of sales")
    sales_clicks: float | None = Field(..., description="The number of clicks for sales")
    sales_promoted: float | None = Field(..., description="The number of promoted sales")
    targeting_expression: str | None = Field(..., description="The expression of the targeting")
    targeting_id: int | None = Field(..., description="The ID of the targeting")
    targeting_text: str | None = Field(..., description="The text of the targeting")
    targeting_type: str | None = Field(..., description="The type of targeting")
    top_of_search_impression_share: float | None = Field(..., description="The impression share at the top of search")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsCampaigns(AssetSchema):
    """
    The Brands Campaigns report provides insights into the performance of Amazon Ads campaigns focused on brand
    promotion. It includes key metrics such as impressions, clicks, conversions, add-to-cart actions, sales, purchases,
    viewability rate, and various new-to-brand metrics.
    """

    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of add to cart clicks")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of branded searches clicks")
    campaign_budget_amount: float | None = Field(..., description="The budget amount for the campaign")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code for the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_rule_based_budget_amount: float | None = Field(
        ..., description="The budget amount based on campaign rules"
    )
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_type: str | None = Field(..., description="The type of cost")
    date: datetime.date | None = Field(..., description="The date of the campaign")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of detail page views clicks")
    ecp_add_to_cart: float | None = Field(..., description="The eCP (Effective Cost per Add to Cart)")
    impressions: float | None = Field(..., description="The number of impressions")
    kindle_edition_normalized_pages_read_14d: float | None = Field(
        ..., description="The number of Kindle Edition Normalized Pages read in the last 14 days"
    )
    kindle_edition_normalized_pages_royalties_14d: float | None = Field(
        ..., description="The royalties for Kindle Edition Normalized Pages read in the last 14 days"
    )
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views_clicks: float | None = Field(
        ..., description="The number of new-to-brand detail page views clicks"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(
        ..., description="The ECP (Effective Cost per Detail Page View) for new-to-brand"
    )
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(..., description="The number of new-to-brand purchases clicks")
    new_to_brand_purchases_percentage: float | None = Field(..., description="The percentage of new-to-brand purchases")
    new_to_brand_purchases_rate: float | None = Field(..., description="The rate of new-to-brand purchases")
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of new-to-brand sales clicks")
    new_to_brand_sales_percentage: float | None = Field(..., description="The percentage of new-to-brand sales")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new-to-brand units sold")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of new-to-brand units sold clicks"
    )
    new_to_brand_units_sold_percentage: float | None = Field(
        ..., description="The percentage of new-to-brand units sold"
    )
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of purchases clicks")
    purchases_promoted: float | None = Field(..., description="The number of promoted purchases")
    sales: float | None = Field(..., description="The total sales")
    sales_clicks: float | None = Field(..., description="The number of sales clicks")
    sales_promoted: float | None = Field(..., description="The number of promoted sales")
    top_of_search_impression_share: float | None = Field(..., description="The impression share at the top of search")
    units_sold: float | None = Field(..., description="The number of units sold")
    units_sold_clicks: float | None = Field(..., description="The number of units sold clicks")
    video_5_second_view_rate: float | None = Field(..., description="The rate of video 5-second views")
    video_5_second_views: float | None = Field(..., description="The number of video 5-second views")
    video_complete_views: float | None = Field(..., description="The number of complete video views")
    video_first_quartile_views: float | None = Field(..., description="The number of video first quartile views")
    video_midpoint_views: float | None = Field(..., description="The number of video midpoint views")
    video_third_quartile_views: float | None = Field(..., description="The number of video third quartile views")
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The rate of view click-through")
    viewability_rate: float | None = Field(..., description="The rate of viewability")
    viewable_impressions: float | None = Field(..., description="The number of viewable impressions")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayPurchasedProducts(AssetSchema):
    """
    The Display Purchased Products report provides insights into the performance of promoted products within
    ad campaigns on Amazon Ads. It includes key metrics such as conversions, sales, and units sold.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    asin_brand_halo: str | None = Field(..., description="The ASIN with brand halo")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    conversions_brand_halo: float | None = Field(..., description="The number of conversions with brand halo")
    conversions_brand_halo_clicks: float | None = Field(
        ..., description="The number of conversions with brand halo clicks"
    )
    date: datetime.date | None = Field(..., description="The date of the record")
    promoted_asin: str | None = Field(..., description="The ASIN of the promoted product")
    promoted_sku: str | None = Field(..., description="The SKU of the promoted product")
    sales_brand_halo: float | None = Field(..., description="The sales with brand halo")
    sales_brand_halo_clicks: float | None = Field(..., description="The sales with brand halo clicks")
    units_sold_brand_halo: float | None = Field(..., description="The number of units sold with brand halo")
    units_sold_brand_halo_clicks: float | None = Field(
        ..., description="The number of units sold with brand halo clicks"
    )



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayGrossAndInvalidTraffic(AssetSchema):
    """
    The Display Gross and Invalid Traffic report provides insights into the quality of traffic generated by campaigns
    on Amazon Ads. It includes key metrics such as total clicks, impressions, gross click-throughs, gross impressions,
    invalid click-through rate, and invalid impressions.
    """

    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    date: datetime.date | None = Field(..., description="The date of the data entry")
    gross_click_throughs: float | None = Field(..., description="The number of gross click throughs")
    gross_impressions: float | None = Field(..., description="The number of gross impressions")
    impressions: float | None = Field(..., description="The number of impressions")
    invalid_click_through_rate: float | None = Field(..., description="The rate of invalid click throughs")
    invalid_click_throughs: float | None = Field(..., description="The number of invalid click throughs")
    invalid_impression_rate: float | None = Field(..., description="The rate of invalid impressions")
    invalid_impressions: float | None = Field(..., description="The number of invalid impressions")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class ProductsSearchTerms(AssetSchema):
    """
    The Products Search Terms report provides insights into the performance of search terms used in advertising
    campaigns. It includes key metrics such as click-through rate, clicks, cost, impressions, sales, purchases,
    keyword details, match types, and targeting criteria.
    """

    acos_clicks_14d: float | None = Field(..., description="The advertising cost of sales for clicks within 14 days")
    acos_clicks_7d: float | None = Field(..., description="The advertising cost of sales for clicks within 7 days")
    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_keyword_status: str | None = Field(..., description="The status of the ad keyword")
    attributed_sales_same_sku_14d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 14 days"
    )
    attributed_sales_same_sku_1d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 1 day"
    )
    attributed_sales_same_sku_30d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 30 days"
    )
    attributed_sales_same_sku_7d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 7 days"
    )
    campaign_budget_amount: float | None = Field(..., description="The budget amount for the campaign")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code for the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    click_through_rate: float | None = Field(..., description="The click-through rate")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost of the ad")
    cost_per_click: float | None = Field(..., description="The cost per click")
    date: datetime.date | None = Field(..., description="The date of the record")
    impressions: float | None = Field(..., description="The number of impressions")
    keyword: str | None = Field(..., description="The keyword used for the ad")
    keyword_bid: float | None = Field(..., description="The bid for the keyword")
    keyword_id: int | None = Field(..., description="The ID of the keyword")
    keyword_type: str | None = Field(..., description="The type of the keyword")
    kindle_edition_normalized_pages_read_14d: float | None = Field(
        ..., description="The pages read for Kindle Edition Normalized Pages within 14 days"
    )
    kindle_edition_normalized_pages_royalties_14d: float | None = Field(
        ..., description="The royalties for Kindle Edition Normalized Pages within 14 days"
    )
    match_type: str | None = Field(..., description="The match type of the keyword")
    portfolio_id: int | None = Field(..., description="The ID of the portfolio")
    purchases_14d: float | None = Field(..., description="The purchases within 14 days")
    purchases_1d: float | None = Field(..., description="The purchases within 1 day")
    purchases_30d: float | None = Field(..., description="The purchases within 30 days")
    purchases_7d: float | None = Field(..., description="The purchases within 7 days")
    purchases_same_sku_14d: float | None = Field(..., description="The purchases for the same SKU within 14 days")
    purchases_same_sku_1d: float | None = Field(..., description="The purchases for the same SKU within 1 day")
    purchases_same_sku_30d: float | None = Field(..., description="The purchases for the same SKU within 30 days")
    purchases_same_sku_7d: float | None = Field(..., description="The purchases for the same SKU within 7 days")
    roas_clicks_14d: float | None = Field(..., description="The return on ad spend for clicks within 14 days")
    roas_clicks_7d: float | None = Field(..., description="The return on ad spend for clicks within 7 days")
    sales_14d: float | None = Field(..., description="The sales within 14 days")
    sales_1d: float | None = Field(..., description="The sales within 1 day")
    sales_30d: float | None = Field(..., description="The sales within 30 days")
    sales_7d: float | None = Field(..., description="The sales within 7 days")
    sales_other_sku_7d: float | None = Field(..., description="The sales for other SKUs within 7 days")
    search_term: str | None = Field(..., description="The search term used for the ad")
    targeting: str | None = Field(..., description="The targeting used for the ad")
    units_sold_clicks_14d: float | None = Field(..., description="The units sold for clicks within 14 days")
    units_sold_clicks_1d: float | None = Field(..., description="The units sold for clicks within 1 day")
    units_sold_clicks_30d: float | None = Field(..., description="The units sold for clicks within 30 days")
    units_sold_clicks_7d: float | None = Field(..., description="The units sold for clicks within 7 days")
    units_sold_other_sku_7d: float | None = Field(..., description="The units sold for other SKUs within 7 days")
    units_sold_same_sku_14d: float | None = Field(..., description="The units sold for the same SKU within 14 days")
    units_sold_same_sku_1d: float | None = Field(..., description="The units sold for the same SKU within 1 day")
    units_sold_same_sku_30d: float | None = Field(..., description="The units sold for the same SKU within 30 days")
    units_sold_same_sku_7d: float | None = Field(..., description="The units sold for the same SKU within 7 days")



import interloper as il
import pandas as pd


@il.source(tags=["Search"])
class SearchConsole:
    @il.asset(tags=["Report"])
    def site(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Website performance in search results including clicks, impressions, CTR, position, and search queries."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def site_by_country_device(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Website performance in search results by country and device including clicks, impressions, CTR, and
        position.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def site_by_country_page(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Website performance in search results by country and page
        including clicks, impressions, CTR, and position.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def search_appearance(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Website appearance in Google Search results including clicks, impressions, CTR, and position."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def page(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Page-level performance in search results including clicks, impressions, CTR, position, and search queries."""

        raise NotImplementedError



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsSearchTerms(AssetSchema):
    """
    The Brands Search Terms report provides insights into the performance of ad campaigns. It includes key metrics such
    as clicks, impressions, purchases, and sales, along with dimensions like ad group, campaign, keyword, and search
    terms.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    campaign_budget_amount: float | None = Field(..., description="The amount of the campaign budget")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost of the ad")
    cost_type: str | None = Field(..., description="The type of cost")
    date: datetime.date | None = Field(..., description="The date of the search term data")
    impressions: float | None = Field(..., description="The number of impressions")
    keyword_bid: float | None = Field(..., description="The bid amount for the keyword")
    keyword_id: int | None = Field(..., description="The ID of the keyword")
    keyword_text: str | None = Field(..., description="The text of the keyword")
    match_type: str | None = Field(..., description="The type of match for the keyword")
    purchases: float | None = Field(..., description="The number of purchases made")
    purchases_clicks: float | None = Field(..., description="The number of purchases made from clicks")
    sales: float | None = Field(..., description="The total sales amount")
    sales_clicks: float | None = Field(..., description="The number of sales made from clicks")
    search_term: str | None = Field(..., description="The search term used")
    units_sold: float | None = Field(..., description="The number of units sold")
    video_5_second_view_rate: float | None = Field(
        ..., description="The rate at which viewers watched at least 5 seconds of the video"
    )
    video_5_second_views: float | None = Field(
        ..., description="The number of views for at least 5 seconds of the video"
    )
    video_complete_views: float | None = Field(..., description="The number of complete video views")
    video_first_quartile_views: float | None = Field(
        ..., description="The number of views for at least 25% of the video"
    )
    video_midpoint_views: float | None = Field(..., description="The number of views for at least 50% of the video")
    video_third_quartile_views: float | None = Field(
        ..., description="The number of views for at least 75% of the video"
    )
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The rate at which viewers clicked through the ad")
    viewability_rate: float | None = Field(..., description="The rate at which the ad was viewable")
    viewable_impressions: float | None = Field(..., description="The number of viewable impressions")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class DisplayVideo360:
    @il.asset(tags=["Report"])
    def line_items(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertising campaign performance and costs including clicks, impressions, and media costs."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def line_items_by_country(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertising campaign performance and costs segmented by country including clicks, impressions, and media
        costs.
        """

        raise NotImplementedError



import interloper as il
import pandas as pd


@il.source(tags=["Social"])
class InstagramInsights:
    @il.asset(tags=["Report"])
    def insights(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Instagram account performance including follower count, impressions, reach, and engagement actions."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def insights_by_age_gender(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Demographic breakdowns of Instagram account insights by age and gender."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def insights_by_country(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Geographic breakdowns of Instagram account insights by country."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def insights_by_city(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Geographic breakdowns of Instagram account insights by city."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def engagement(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Instagram engagement metrics."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def media(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Instagram media performance report."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def profiles(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Instagram profile data."""

        raise NotImplementedError



import logging

import interloper as il
import pandas as pd
from pydantic_settings import SettingsConfigDict

logger = logging.getLogger(__name__)


class DemoSchema(il.AssetSchema):
    hello: str


class DemoConfig(il.Config):
    hello: str = "world"

    model_config = SettingsConfigDict(env_prefix="Demo_")


partitioning = il.TimePartitionConfig(column="date", allow_window=False)


@il.source(
    config=DemoConfig,
    tags=["Testing"],
)
class DemoSource:
    """Demo source. Defines a small DAG (a -> b,c,d -> e) with time partitioning."""

    config: DemoConfig

    def do(self) -> None:
        import random
        import time

        time.sleep(random.uniform(0.5, 1.5))
        if random.random() < 0.15:
            raise RuntimeError("Random failure in demo source")

    @il.asset(
        schema=DemoSchema,
        partitioning=partitioning,
        tags=["Report"],
    )
    def a(
        self,
        context: il.ExecutionContext,
    ) -> pd.DataFrame:
        """Root asset. Returns a single row with the configured greeting."""
        print(f"Hello from A {self.config.hello}")
        self.do()
        return pd.DataFrame([{"hello": self.config.hello}])

    @il.asset(
        schema=DemoSchema,
        partitioning=partitioning,
        tags=["Report"],
    )
    def b(
        self,
        context: il.ExecutionContext,
        a: str,
    ) -> pd.DataFrame:
        """Depends on A. Part of the example DAG (a -> b -> e)."""
        print(f"Hello from B {self.config.hello}")
        self.do()
        return pd.DataFrame([{"hello": self.config.hello}])

    @il.asset(
        schema=DemoSchema,
        partitioning=partitioning,
        tags=["Report"],
    )
    def c(
        self,
        context: il.ExecutionContext,
        a: str,
    ) -> pd.DataFrame:
        """Depends on A. Part of the example DAG (a -> c -> e)."""
        print(f"Hello from C {self.config.hello}")
        self.do()
        return pd.DataFrame([{"hello": self.config.hello}])

    @il.asset(
        schema=DemoSchema,
        partitioning=partitioning,
        tags=["Report"],
    )
    def d(
        self,
        context: il.ExecutionContext,
        a: str,
    ) -> pd.DataFrame:
        """Depends on A. Part of the example DAG (a -> d -> e)."""
        print(f"Hello from D {self.config.hello}")
        self.do()
        return pd.DataFrame([{"hello": self.config.hello}])

    @il.asset(
        schema=DemoSchema,
        partitioning=partitioning,
        tags=["Report"],
    )
    def e(
        self,
        context: il.ExecutionContext,
        b: str,
        c: str,
        d: str,
    ) -> pd.DataFrame:
        """Depends on B, C, and D. Sink asset of the example DAG."""
        print(f"Hello from E {self.config.hello}")
        self.do()
        return pd.DataFrame([{"hello": self.config.hello}])


#    ↗ b ↘
#  a → c → e
#    ↘ d ↗



from interloper.schema import AssetSchema
from pydantic import Field


class ConversionsByTimeOfDay(AssetSchema):
    """
    The Conversions by Time of Day report provides insights into conversion patterns based on the day of the week
    and hour of the day. It includes key metrics such as the number of conversions and revenue generated within each
    time segment.
    """

    conversions: int = Field(description="Number of conversions")
    day: str = Field(description="Day of the week")
    hour: int = Field(description="Hour of the day")
    revenue: float = Field(description="Revenue generated")



import datetime as dt

import interloper as il
import pandas as pd
from pydantic_settings import SettingsConfigDict

from interloper_assets.adup import constants, schemas


class AdupConfig(il.Config):
    client_id: str
    client_secret: str

    model_config = SettingsConfigDict(env_prefix="adup_")


@il.source(
    config=AdupConfig,
    tags=["Advertising"],
)
class Adup:
    """Adup advertising platform integration."""

    def __init__(self, config: AdupConfig) -> None:
        auth = il.OAuth2ClientCredentialsAuth(constants.BASE_URL, config.client_id, config.client_secret)
        self.client = il.RESTClient(constants.BASE_URL, auth)

    def get_report(self, report_type: str, start_date: dt.date, end_date: dt.date) -> dict:
        response = self.client.post(
            "/reports/v202101/report",
            json={
                "report_name": report_type,
                "report_type": report_type,
                "select": constants.FIELDS[report_type],
                "conditions": [],
                "download_format": "JSON",
                "date_range_type": "CUSTOM_DATE",
                "date_range": {
                    "min": start_date.isoformat(),
                    "max": end_date.isoformat(),
                },
            },
        )
        response.raise_for_status()

        return response.json()

    @il.asset(
        schema=schemas.Account,
        tags=["Entity"],
    )
    def account(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertiser account information."""

        response = self.client.get("/advertisers/me")
        response.raise_for_status()
        data = response.json()
        return pd.DataFrame([data])

    @il.asset(
        schema=schemas.Ads,
        partitioning=il.TimePartitionConfig(column="Date", allow_window=True),
        tags=["Report"],
    )
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance insights with metrics like impressions, clicks, conversions, and cost."""

        (start_date, end_date) = context.partition_date_window
        response = self.get_report("AD_PERFORMANCE_REPORT", start_date, end_date)
        data = response["rows"]
        return pd.DataFrame(data)



import interloper as il
import pandas as pd


@il.source(tags=["Social"])
class FacebookInsights:
    @il.asset(tags=["Report"])
    def page(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Engagement and reach metrics for a Facebook page including reactions, impressions, engagements, and video
        views.
        """

        raise NotImplementedError



import datetime as dt
import logging

import httpx
import interloper as il
import pandas as pd
from pydantic_settings import SettingsConfigDict

from interloper_assets.adservice import schemas

logger = logging.getLogger(__name__)


class AdserviceConfig(il.Config):
    api_key: str

    model_config = SettingsConfigDict(env_prefix="adservice_")


@il.source(
    config=AdserviceConfig,
    tags=["Advertising"],
)
class Adservice:
    """Adservice advertising platform integration."""

    def __init__(self, config: AdserviceConfig) -> None:
        base_url = "https://api.adservice.com/v2/client"
        auth = httpx.BasicAuth(username="api", password=config.api_key)
        self.client = il.RESTClient(base_url, auth)
        self.base_url = base_url

    def get_report(
        self,
        start_date: dt.date,
        end_date: dt.date,
        report_type: str,
        group_by: str | None = None,
        end_group: str | None = None,
        sales_amount: int | None = None,
    ) -> dict:
        response = self.client.get(
            url=f"{self.base_url}/{report_type}",
            params={
                "from_date": start_date.isoformat(),
                "to_date": end_date.isoformat(),
                "sales_amount": sales_amount,
                "group_by": group_by,
                "end_group": end_group,
            },
        )
        response.raise_for_status()
        return response.json()

    @il.asset(
        schema=schemas.Campaigns,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance statistics with metrics like impressions, clicks, and conversions."""

        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="statistics",
            group_by="stamp,camp_id",
            end_group="stamp",
            sales_amount=1,
        )
        data = response["data"]["rows"]
        return pd.DataFrame(data)

    @il.asset(
        schema=schemas.ConversionsReport,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def conversions(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Conversion events and attribution data."""
        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="conversions",
        )
        data = response["data"]
        return pd.DataFrame(data)

    @il.asset(
        schema=schemas.ConversionsByTimeOfDay,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def conversions_time_of_day(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Conversion events broken down by time of day."""

        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="statistics/conversions/timeofday",
        )
        data = response["data"]
        return pd.DataFrame(data)

    @il.asset(
        schema=schemas.CampaignsByCity,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def campaigns_by_city(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance segmented by city."""

        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="statistics/devicedetails",
            group_by="city",
        )
        data = response["data"]
        return pd.DataFrame(data)

    @il.asset(
        schema=schemas.CampaignsByBrowser,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def campaigns_by_browser(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance segmented by browser."""

        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="statistics/devicedetails",
            group_by="browser",
        )
        data = response["data"]
        return pd.DataFrame(data)

    @il.asset(
        schema=schemas.CampaignsByDeviceType,
        partitioning=il.TimePartitionConfig(column="date"),
    )
    def campaigns_by_device_type(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance segmented by device type."""

        response = self.get_report(
            start_date=context.partition_date,
            end_date=context.partition_date,
            report_type="statistics/devicedetails",
            group_by="device_type",
        )
        data = response["data"]
        return pd.DataFrame(data)



from interloper_assets.adservice.schemas.campaigns import Campaigns
from interloper_assets.adservice.schemas.campaigns_by_browser import CampaignsByBrowser
from interloper_assets.adservice.schemas.campaigns_by_city import CampaignsByCity
from interloper_assets.adservice.schemas.campaigns_by_device_type import CampaignsByDeviceType
from interloper_assets.adservice.schemas.conversions import ConversionsReport
from interloper_assets.adservice.schemas.conversions_by_time_of_day import ConversionsByTimeOfDay

__all__ = [
    "Campaigns",
    "CampaignsByCity",
    "CampaignsByDeviceType",
    "CampaignsByBrowser",
    "ConversionsReport",
    "ConversionsByTimeOfDay",
]



BASE_URL = "https://api.adup-tech.com"

FIELDS = {
    "ACCOUNT_PERFORMANCE_REPORT": [
        "AccountDescriptiveName",
        "AccountId",
        "AllConversions",
        "AverageCost",
        "AverageCpc",
        "AverageCpm",
        "AveragePosition",
        "Clicks",
        "ConversionRate",
        "Conversions",
        "Cost",
        "CostPerAllConversion",
        "CostPerConversion",
        "Ctr",
        "Date",
        "Impressions",
        "InteractionRate",
        "Interactions",
        "InvalidClickRate",
        "InvalidClicks",
        "ViewThroughConversions",
    ],
    "AD_PERFORMANCE_REPORT": [
        "AccountDescriptiveName",
        "AccountId",
        "AdgroupId",
        "AdgroupName",
        "AdgroupStatus",
        "AdName",
        "AllConversions",
        "AverageCost",
        "AverageCpc",
        "AverageCpm",
        "AveragePosition",
        "CampaignId",
        "CampaignName",
        "CampaignStatus",
        "Clicks",
        "ConversionRate",
        "Conversions",
        "Cost",
        "CostPerAllConversion",
        "CostPerConversion",
        "CreativeFinalMobileUrl",
        "CreativeFinalUrl",
        "CreativeFinalUrlSuffix",
        "Ctr",
        "Date",
        "Description",
        "DisplayUrl",
        "Headline",
        "Impressions",
        "InteractionRate",
        "Interactions",
        "Path1",
        "Path2",
        "ViewThroughConversions",
    ],
    "CAMPAIGN_PERFORMANCE_REPORT": [
        "AccountDescriptiveName",
        "AccountId",
        "AllConversionRate",
        "AllConversions",
        "AverageCost",
        "AverageCpc",
        "AverageCpm",
        "AveragePosition",
        "CampaignDesktopBidModifier",
        "CampaignId",
        "CampaignMobileBidModifier",
        "CampaignName",
        "CampaignStatus",
        "CampaignTabletBidModifier",
        "Clicks",
        "ConversionRate",
        "Conversions",
        "Cost",
        "CostPerAllConversion",
        "CostPerConversion",
        "Ctr",
        "Date",
        "Device",
        "EndDate",
        "Impressions",
        "InteractionRate",
        "Interactions",
        "InvalidClickRate",
        "InvalidClicks",
        "Period",
        "StartDate",
        "TrackingUrlTemplate",
        "ViewThroughConversions",
    ],
    "DIRECT_PLACEMENT_PERFORMANCE_REPORT": [
        "AccountDescriptiveName",
        "AccountId",
        "AdgroupName",
        "AdgroupStatus",
        "AllConversions",
        "AverageCost",
        "AverageCpc",
        "AverageCpm",
        "AveragePosition",
        "CampaignId",
        "CampaignName",
        "CampaignStatus",
        "Clicks",
        "ConversionRate",
        "Conversions",
        "Cost",
        "CostPerAllConversion",
        "CostPerConversion",
        "Ctr",
        "Date",
        "DisplayName",
        "Id",
        "Impressions",
        "InteractionRate",
        "Interactions",
        "ViewThroughConversions",
    ],
    "PLACEMENTLAYOUT_PERFORMANCE_REPORT": [
        "AverageContainerCpm",
        "AverageCpc",
        "AverageCpm",
        "Clicks",
        "ContainerCtr",
        "ContainerImpressions",
        "ContainerInvalidClickRate",
        "Ctr",
        "Date",
        "Impressions",
        "InvalidClickRate",
        "InvalidClicks",
        "PlacementId",
        "PlacementLayoutId",
        "PlacementLayoutName",
        "PlacementLayoutStatus",
        "PlacementName",
        "Revenue",
        "SiteId",
        "SiteName",
    ],
}



from interloper.schema import AssetSchema
from pydantic import Field


class Profiles(AssetSchema):
    profile_id: int = Field(..., description="The Amazon Ads profile ID")
    country_code: str = Field(..., description="The account's country code")
    currency_code: str = Field(..., description="The account's currency code")
    timezone: str = Field(..., description="The account's timezone")
    account_info_marketplace_string_id: str = Field(..., description="Account info marketplace string ID")
    account_info_id: str = Field(..., description="Account info id")
    account_info_type: str = Field(..., description="Account info type")
    account_info_name: str = Field(..., description="Account info name")
    account_info_valid_payment_method: bool = Field(..., description="Whether account's payment method is valid")


import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class ProductsAdvertisedProducts(AssetSchema):
    """
    The Products Advertised Products report provides insights into the performance of advertised products on Amazon Ads.
    It includes key metrics such as clicks, impressions, sales, purchases, and return on ad spend (ROAS),
    as well as dimensions like ad group name, campaign name, and SKU.
    """

    acos_clicks_14d: float | None = Field(
        ..., description="The advertising cost of sales (ACoS) for clicks within 14 days"
    )
    acos_clicks_7d: float | None = Field(
        ..., description="The advertising cost of sales (ACoS) for clicks within 7 days"
    )
    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_id: int | None = Field(..., description="The ID of the ad")
    advertised_asin: str | None = Field(..., description="The ASIN of the advertised product")
    advertised_sku: str | None = Field(..., description="The SKU of the advertised product")
    attributed_sales_same_sku_14d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 14 days"
    )
    attributed_sales_same_sku_1d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 1 day"
    )
    attributed_sales_same_sku_30d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 30 days"
    )
    attributed_sales_same_sku_7d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 7 days"
    )
    campaign_budget_amount: float | None = Field(..., description="The budget amount of the campaign")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    click_through_rate: float | None = Field(..., description="The click-through rate (CTR)")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_per_click: float | None = Field(..., description="The cost per click")
    date: datetime.date | None = Field(..., description="The date of the record")
    impressions: float | None = Field(..., description="The number of impressions")
    kindle_edition_normalized_pages_read_14d: float | None = Field(
        ..., description="The pages read for Kindle edition normalized pages within 14 days"
    )
    kindle_edition_normalized_pages_royalties_14d: float | None = Field(
        ..., description="The royalties for Kindle edition normalized pages within 14 days"
    )
    portfolio_id: int | None = Field(..., description="The ID of the portfolio")
    purchases_14d: float | None = Field(..., description="The purchases within 14 days")
    purchases_1d: float | None = Field(..., description="The purchases within 1 day")
    purchases_30d: float | None = Field(..., description="The purchases within 30 days")
    purchases_7d: float | None = Field(..., description="The purchases within 7 days")
    purchases_same_sku_14d: float | None = Field(..., description="The purchases for the same SKU within 14 days")
    purchases_same_sku_1d: float | None = Field(..., description="The purchases for the same SKU within 1 day")
    purchases_same_sku_30d: float | None = Field(..., description="The purchases for the same SKU within 30 days")
    purchases_same_sku_7d: float | None = Field(..., description="The purchases for the same SKU within 7 days")
    roas_clicks_14d: float | None = Field(..., description="The return on ad spend (ROAS) for clicks within 14 days")
    roas_clicks_7d: float | None = Field(..., description="The return on ad spend (ROAS) for clicks within 7 days")
    sales_14d: float | None = Field(..., description="The sales within 14 days")
    sales_1d: float | None = Field(..., description="The sales within 1 day")
    sales_30d: float | None = Field(..., description="The sales within 30 days")
    sales_7d: float | None = Field(..., description="The sales within 7 days")
    sales_other_sku_7d: float | None = Field(..., description="The sales for other SKUs within 7 days")
    spend: float | None = Field(..., description="The amount spent")
    units_sold_clicks_14d: float | None = Field(..., description="The units sold for clicks within 14 days")
    units_sold_clicks_1d: float | None = Field(..., description="The units sold for clicks within 1 day")
    units_sold_clicks_30d: float | None = Field(..., description="The units sold for clicks within 30 days")
    units_sold_clicks_7d: float | None = Field(..., description="The units sold for clicks within 7 days")
    units_sold_other_sku_7d: float | None = Field(..., description="The units sold for other SKUs within 7 days")
    units_sold_same_sku_14d: float | None = Field(..., description="The units sold for the same SKU within 14 days")
    units_sold_same_sku_1d: float | None = Field(..., description="The units sold for the same SKU within 1 day")
    units_sold_same_sku_30d: float | None = Field(..., description="The units sold for the same SKU within 30 days")
    units_sold_same_sku_7d: float | None = Field(..., description="The units sold for the same SKU within 7 days")



import interloper as il
import pandas as pd

from interloper_assets.amazon_ads.source import AmazonAds


@il.source(tags=["Analytics"])
class CampaignPerformanceAnalysis:
    """Cross-source campaign matching and advanced performance analytics."""

    @il.asset(
        tags=["Transform"],
        requires={
            "amazon_ads_campaigns": AmazonAds.products_campaigns.definition_key,
        },
    )
    def matcher(
        self,
        context: il.ExecutionContext,
        amazon_ads_campaigns: pd.DataFrame,
    ) -> pd.DataFrame:
        """Matches campaigns from different advertising sources under the same bucket."""

        raise NotImplementedError

    @il.asset(
        tags=["Analytics"],
    )
    def performance_analysis(
        self,
        context: il.ExecutionContext,
        matcher: pd.DataFrame,
    ) -> pd.DataFrame:
        """Advanced analytics of campaign performance across all matched campaigns."""

        raise NotImplementedError



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class ProductsCampaigns(AssetSchema):
    """
    The Products Campaigns report provides insights into the performance of campaigns for advertised products on
    Amazon Ads. It includes key metrics such as clicks, impressions, sales, purchases, and attributed sales, along with
    dimensions like campaign name, ad group name, and campaign status.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_status: str | None = Field(..., description="The status of the ad")
    attributed_sales_same_sku_14d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 14 days"
    )
    attributed_sales_same_sku_1d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 1 day"
    )
    attributed_sales_same_sku_30d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 30 days"
    )
    attributed_sales_same_sku_7d: float | None = Field(
        ..., description="The attributed sales for the same SKU within 7 days"
    )
    campaign_applicable_budget_rule_name: float | None = Field(
        ..., description="The name of the applicable budget rule for the campaign"
    )
    campaign_applicable_budget_ruleid: int | None = Field(
        ..., description="The ID of the applicable budget rule for the campaign"
    )
    campaign_bidding_strategy: str | None = Field(..., description="The bidding strategy of the campaign")
    campaign_budget_amount: float | None = Field(..., description="The amount of campaign budget")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_rule_based_budget_amount: float | None = Field(
        ..., description="The budget amount based on campaign rules"
    )
    campaign_status: str | None = Field(..., description="The status of the campaign")
    click_through_rate: float | None = Field(..., description="The click-through rate")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_per_click: float | None = Field(..., description="The cost per click")
    date: datetime.date | None = Field(..., description="The date of the campaign")
    impressions: float | None = Field(..., description="The number of impressions")
    kindle_edition_normalized_pages_read_14d: float | None = Field(
        ..., description="The number of Kindle edition normalized pages read within 14 days"
    )
    kindle_edition_normalized_pages_royalties_14d: float | None = Field(
        ..., description="The royalties for Kindle edition normalized pages within 14 days"
    )
    purchases_14d: float | None = Field(..., description="The purchases within 14 days")
    purchases_1d: float | None = Field(..., description="The purchases within 1 day")
    purchases_30d: float | None = Field(..., description="The purchases within 30 days")
    purchases_7d: float | None = Field(..., description="The purchases within 7 days")
    purchases_same_sku_14d: float | None = Field(..., description="The purchases for the same SKU within 14 days")
    purchases_same_sku_1d: float | None = Field(..., description="The purchases for the same SKU within 1 day")
    purchases_same_sku_30d: float | None = Field(..., description="The purchases for the same SKU within 30 days")
    purchases_same_sku_7d: float | None = Field(..., description="The purchases for the same SKU within 7 days")
    sales_14d: float | None = Field(..., description="The sales within 14 days")
    sales_1d: float | None = Field(..., description="The sales within 1 day")
    sales_30d: float | None = Field(..., description="The sales within 30 days")
    sales_7d: float | None = Field(..., description="The sales within 7 days")
    spend: float | None = Field(..., description="The amount spent")
    units_sold_clicks_14d: float | None = Field(..., description="The units sold through clicks within 14 days")
    units_sold_clicks_1d: float | None = Field(..., description="The units sold through clicks within 1 day")
    units_sold_clicks_30d: float | None = Field(..., description="The units sold through clicks within 30 days")
    units_sold_clicks_7d: int | None = Field(..., description="The units sold through clicks within 7 days")
    units_sold_same_sku_14d: float | None = Field(..., description="The units sold for the same SKU within 14 days")
    units_sold_same_sku_1d: float | None = Field(..., description="The units sold for the same SKU within 1 day")
    units_sold_same_sku_30d: float | None = Field(..., description="The units sold for the same SKU within 30 days")
    units_sold_same_sku_7d: float | None = Field(..., description="The units sold for the same SKU within 7 days")



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayCampaigns(AssetSchema):
    """
    The Display Campaigns report provides insights into the performance of display advertising campaigns.
    It includes key metrics such as clicks, impressions, purchases, sales, viewability rates, and more.
    """

    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of add to cart clicks")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    add_to_cart_views: float | None = Field(..., description="The number of add to cart views")
    branded_search_rate: float | None = Field(..., description="The rate of branded searches")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of branded searches clicks")
    branded_searches_views: float | None = Field(..., description="The number of branded searches views")
    campaign_budget_amount: float | None = Field(..., description="The amount of campaign budget")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_type: str | None = Field(..., description="The cost type")
    cumulative_reach: float | None = Field(..., description="The cumulative reach")
    date: datetime.date | None = Field(..., description="The date of the campaign")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of detail page views clicks")
    ecp_add_to_cart: float | None = Field(..., description="The ECP of add to cart")
    ecp_brand_search: float | None = Field(..., description="The ECP of brand search")
    impressions: float | None = Field(..., description="The number of impressions")
    impressions_frequency_average: float | None = Field(..., description="The average frequency of impressions")
    impressions_views: float | None = Field(..., description="The number of impressions views")
    new_to_brand_detail_page_view_clicks: float | None = Field(
        ..., description="The number of new to brand detail page view clicks"
    )
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new to brand detail page views"
    )
    new_to_brand_detail_page_view_views: float | None = Field(
        ..., description="The number of new to brand detail page view views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of new to brand detail page views"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(..., description="The ECP of new to brand detail page view")
    new_to_brand_purchases: float | None = Field(..., description="The number of new to brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(..., description="The number of new to brand purchases clicks")
    new_to_brand_sales: float | None = Field(..., description="The number of new to brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of new to brand sales clicks")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new to brand units sold")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of new to brand units sold clicks"
    )
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of purchases clicks")
    purchases_promoted_clicks: float | None = Field(..., description="The number of purchases promoted clicks")
    sales: float | None = Field(..., description="The number of sales")
    sales_clicks: float | None = Field(..., description="The number of sales clicks")
    sales_promoted_clicks: float | None = Field(..., description="The number of sales promoted clicks")
    units_sold: float | None = Field(..., description="The number of units sold")
    units_sold_clicks: float | None = Field(..., description="The number of units sold clicks")
    video_complete_views: float | None = Field(..., description="The number of video complete views")
    video_first_quartile_views: float | None = Field(..., description="The number of video first quartile views")
    video_midpoint_views: float | None = Field(..., description="The number of video midpoint views")
    video_third_quartile_views: float | None = Field(..., description="The number of video third quartile views")
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The rate of view click through")
    viewability_rate: float | None = Field(..., description="The rate of viewability")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class BingAds:
    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertising campaign insights including clicks, conversions, impressions, spend, and ad attributes."""

        raise NotImplementedError



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsPlacements(AssetSchema):
    """
    The Brands Placements report provides insights into the performance of Amazon Ads campaigns with a focus on
    different ad placements. It includes key metrics such as clicks, impressions, add-to-cart actions, branded
    searches, detail page views, purchases, sales, units sold, and various engagement metrics related to video ads.
    """

    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of clicks for add to cart")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of clicks for branded searches")
    campaign_budget_amount: float | None = Field(..., description="The amount of campaign budget")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_budget_type: str | None = Field(..., description="The type of campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    campaign_status: str | None = Field(..., description="The status of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cost_type: str | None = Field(..., description="The type of cost")
    date: datetime.date | None = Field(..., description="The date of the data entry")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of clicks for detail page views")
    ecp_add_to_cart: float | None = Field(..., description="The eCP add to cart")
    impressions: float | None = Field(..., description="The number of impressions")
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of new-to-brand detail page views"
    )
    new_to_brand_detail_page_views_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand detail page views"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(
        ..., description="The ECP detail page view for new-to-brand"
    )
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand purchases"
    )
    new_to_brand_purchases_percentage: float | None = Field(..., description="The percentage of new-to-brand purchases")
    new_to_brand_purchases_rate: float | None = Field(..., description="The rate of new-to-brand purchases")
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of clicks for new-to-brand sales")
    new_to_brand_sales_percentage: float | None = Field(..., description="The percentage of new-to-brand sales")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new-to-brand units sold")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand units sold"
    )
    new_to_brand_units_sold_percentage: float | None = Field(
        ..., description="The percentage of new-to-brand units sold"
    )
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of clicks for purchases")
    purchases_promoted: float | None = Field(..., description="The number of promoted purchases")
    sales: float | None = Field(..., description="The total sales")
    sales_clicks: float | None = Field(..., description="The number of clicks for sales")
    sales_promoted: float | None = Field(..., description="The number of promoted sales")
    units_sold: float | None = Field(..., description="The number of units sold")
    units_sold_clicks: float | None = Field(..., description="The number of clicks for units sold")
    video_5_second_view_rate: float | None = Field(..., description="The rate of video 5-second views")
    video_5_second_views: float | None = Field(..., description="The number of video 5-second views")
    video_complete_views: float | None = Field(..., description="The number of complete video views")
    video_first_quartile_views: float | None = Field(..., description="The number of video first quartile views")
    video_midpoint_views: float | None = Field(..., description="The number of video midpoint views")
    video_third_quartile_views: float | None = Field(..., description="The number of video third quartile views")
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The rate of view click-through")
    viewability_rate: float | None = Field(..., description="The rate of viewability")
    viewable_impressions: float | None = Field(..., description="The number of viewable impressions")



from interloper.schema import AssetSchema
from pydantic import Field


class CampaignsByBrowser(AssetSchema):
    """
    The Campaigns by Browser report provides insights into campaign performance based on the user's browser usage.
    It includes key metrics such as conversion rates, the number of conversions, and user demographics such as
    city, country, device model, device type, and operating system.
    """

    agent_id: int = Field(description="The ID of the agent")
    browser: str = Field(description="The browser used by the user")
    camp_id: int = Field(description="The ID of the campaign")
    camp_title: str = Field(description="The title of the campaign")
    city: str = Field(description="The city where the user is located")
    conversion_pct: int = Field(description="The percentage of conversions")
    conversions: int = Field(description="The number of conversions")
    country: str = Field(description="The country where the user is located")
    device_model: str = Field(description="The model of the device used by the user")
    device_type: str = Field(description="The type of device used by the user")
    os: str = Field(description="The operating system used by the user")



from interloper.schema import AssetSchema
from pydantic import Field


class Account(AssetSchema):
    username: str = Field(..., description="The username of the account")
    label: str = Field(..., description="The label of the account")
    type: str = Field(..., description="The type of the account")
    status: str = Field(..., description="The status of the account")
    language: str = Field(..., description="The language used in the account")
    timezone: str = Field(..., description="The timezone of the account")
    id: int = Field(..., description="The advertiser account ID")
    advertiser_url: str = Field(..., description="The advertiser's URL")
    channel_id: int = Field(..., description="The channel ID")


import datetime as dt

from interloper.schema import AssetSchema
from pydantic import Field


class Ads(AssetSchema):
    account_descriptive_name: str = Field(..., description="The descriptive name of the account")
    account_id: int = Field(..., description="The account ID")
    ad_name: str = Field(..., description="The name of the ad")
    adgroup_id: int = Field(..., description="The ad group ID")
    adgroup_name: str = Field(..., description="The name of the ad group")
    adgroup_status: str = Field(..., description="The status of the ad group")
    all_conversions: int = Field(..., description="The total number of conversions")
    average_cost: int = Field(..., description="The average cost")
    average_cpc: int = Field(..., description="The average cost per click")
    average_cpm: int = Field(..., description="The average cost per thousand impressions")
    average_position: float = Field(..., description="The average position")
    campaign_id: int = Field(..., description="The campaign ID")
    campaign_name: str = Field(..., description="The name of the campaign")
    campaign_status: str = Field(..., description="The status of the campaign")
    clicks: int = Field(..., description="The total number of clicks")
    conversion_rate: float = Field(..., description="The conversion rate")
    conversions: int = Field(..., description="The total number of conversions")
    cost: int = Field(..., description="The total cost")
    cost_per_all_conversion: int = Field(..., description="The cost per all conversion")
    cost_per_conversion: int = Field(..., description="The cost per conversion")
    creative_final_mobile_url: str = Field(..., description="The final mobile URL of the creative")
    creative_final_url: str = Field(..., description="The final URL of the creative")
    creative_final_url_suffix: str = Field(..., description="The final URL suffix of the creative")
    ctr: float = Field(..., description="The click-through rate")
    date: dt.date = Field(..., description="The date of the data")
    description: str = Field(..., description="The description")
    display_url: str = Field(..., description="The display URL")
    headline: str = Field(..., description="The headline")
    id: int = Field(..., description="The ID")
    impressions: int = Field(..., description="The total number of impressions")
    interaction_rate: float = Field(..., description="The interaction rate")
    interactions: int = Field(..., description="The total number of interactions")
    path1: str = Field(..., description="The first path")
    path2: str = Field(..., description="The second path")
    view_through_conversions: int = Field(..., description="The total number of view-through conversions")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class SearchAds360:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance on Search Ads 360 including clicks,
        impressions, average cost, and click-through rate.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def daily_campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Daily campaign performance on Search Ads 360 including clicks, impressions, cost, and conversions."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def monthly_campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Monthly campaign performance on Search Ads 360 including clicks, impressions, cost, and conversions."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def customer_clients(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Customer client performance on Search Ads 360 including clicks, impressions, cost, and conversions."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def customer_clients_metadata(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Customer client metadata including customer ID, client level, manager information, and account details."""

        raise NotImplementedError



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayTargeting(AssetSchema):
    """
    The Display Targeting report provides insights into the performance of ad campaigns on Amazon Ads based on
    specific targeting criteria. It includes key metrics such as clicks, impressions, and detail page views, along with
    dimensions like ad group name and targeting expression.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_keyword_status: str | None = Field(..., description="The status of the ad keyword")
    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of clicks for add to cart")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    add_to_cart_views: float | None = Field(..., description="The number of add to cart views")
    branded_search_rate: float | None = Field(..., description="The rate of branded searches")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of clicks for branded searches")
    branded_searches_views: float | None = Field(..., description="The number of views for branded searches")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    date: datetime.date | None = Field(..., description="The date of the data entry")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of clicks for detail page views")
    ecp_add_to_cart: float | None = Field(..., description="The ECP for add to cart")
    ecp_brand_search: float | None = Field(..., description="The ECP for brand search")
    impressions: float | None = Field(..., description="The number of impressions")
    impressions_views: float | None = Field(..., description="The number of views for impressions")
    new_to_brand_detail_page_view_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand detail page views"
    )
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new-to-brand detail page views"
    )
    new_to_brand_detail_page_view_views: float | None = Field(
        ..., description="The number of views for new-to-brand detail page views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of detail page views for new-to-brand"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(
        ..., description="The ECP for new-to-brand detail page views"
    )
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(
        ..., description="The number of clicks for new-to-brand purchases"
    )
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of clicks for new-to-brand sales")
    new_to_brand_units_sold: float | None = Field(..., description="The number of units sold for new-to-brand")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of clicks for units sold for new-to-brand"
    )
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of clicks for purchases")
    purchases_promoted_clicks: float | None = Field(..., description="The number of clicks for purchases promoted")
    sales: float | None = Field(..., description="The number of sales")
    sales_clicks: float | None = Field(..., description="The number of clicks for sales")
    sales_promoted_clicks: float | None = Field(..., description="The number of clicks for sales promoted")
    targeting_expression: str | None = Field(..., description="The expression of the targeting")
    targeting_id: int | None = Field(..., description="The ID of the targeting")
    targeting_text: str | None = Field(..., description="The text of the targeting")
    units_sold: float | None = Field(..., description="The number of units sold")
    units_sold_clicks: float | None = Field(..., description="The number of clicks for units sold")
    video_complete_views: float | None = Field(..., description="The number of complete video views")
    video_first_quartile_views: float | None = Field(..., description="The number of views for video first quartile")
    video_midpoint_views: float | None = Field(..., description="The number of views for video midpoint")
    video_third_quartile_views: float | None = Field(..., description="The number of views for video third quartile")
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The click-through rate for views")
    viewability_rate: float | None = Field(..., description="The rate of viewability")



import interloper as il
from interloper.errors import SourceError

from interloper_assets.adservice.source import Adservice, AdserviceConfig
from interloper_assets.adup.source import Adup, AdupConfig
from interloper_assets.amazon_ads.source import AmazonAds, AmazonAdsConfig
from interloper_assets.amazon_selling_partner.source import AmazonSellingPartner
from interloper_assets.awin.source import Awin
from interloper_assets.bing_ads.source import BingAds
from interloper_assets.campaign_manager_360.source import CampaignManager360
from interloper_assets.campaign_performance_analysis.source import CampaignPerformanceAnalysis
from interloper_assets.criteo.source import Criteo
from interloper_assets.demo.source import DemoConfig, DemoSource
from interloper_assets.display_video_360.source import DisplayVideo360
from interloper_assets.facebook_ads.source import FacebookAds
from interloper_assets.facebook_insights.source import FacebookInsights
from interloper_assets.instagram_insights.source import InstagramInsights
from interloper_assets.linkedin_ads.source import LinkedinAds
from interloper_assets.linkedin_organic.source import LinkedinOrganic
from interloper_assets.pinterest_ads.source import PinterestAds
from interloper_assets.search_ads_360.source import SearchAds360
from interloper_assets.search_console.source import SearchConsole
from interloper_assets.snapchat_ads.source import SnapchatAds
from interloper_assets.teads.source import Teads
from interloper_assets.thetradedesk.source import TheTradeDesk
from interloper_assets.tiktok_ads.source import TiktokAds

SOURCE_REGISTRY: dict[str, tuple[il.SourceDefinition, type[il.Config] | None]] = {
    DemoSource.name: (DemoSource, DemoConfig),
    Adup.name: (Adup, AdupConfig),
    Adservice.name: (Adservice, AdserviceConfig),
    AmazonAds.name: (AmazonAds, AmazonAdsConfig),
    AmazonSellingPartner.name: (AmazonSellingPartner, None),
    Awin.name: (Awin, None),
    BingAds.name: (BingAds, None),
    CampaignManager360.name: (CampaignManager360, None),
    CampaignPerformanceAnalysis.name: (CampaignPerformanceAnalysis, None),
    Criteo.name: (Criteo, None),
    DisplayVideo360.name: (DisplayVideo360, None),
    FacebookAds.name: (FacebookAds, None),
    FacebookInsights.name: (FacebookInsights, None),
    InstagramInsights.name: (InstagramInsights, None),
    LinkedinAds.name: (LinkedinAds, None),
    LinkedinOrganic.name: (LinkedinOrganic, None),
    PinterestAds.name: (PinterestAds, None),
    SearchAds360.name: (SearchAds360, None),
    SearchConsole.name: (SearchConsole, None),
    SnapchatAds.name: (SnapchatAds, None),
    Teads.name: (Teads, None),
    TheTradeDesk.name: (TheTradeDesk, None),
    TiktokAds.name: (TiktokAds, None),
}


def get_source_and_config(id: str) -> tuple[il.SourceDefinition, type[il.Config] | None]:
    """Get a source definition and its config type by source type ID.

    Args:
        id: Source type identifier (e.g. FacebookAds.name, DemoSource.name)

    Returns:
        Tuple of (SourceDefinition, ConfigType or None)

    Raises:
        ValueError: If the source ID is not found in the registry
    """
    if id not in SOURCE_REGISTRY:
        raise SourceError(f"Unknown source ID: {id}")
    return SOURCE_REGISTRY[id]


def get_all_sources() -> dict[str, tuple[il.SourceDefinition, type[il.Config] | None]]:
    """Get all registered sources.

    Returns:
        Dictionary mapping source type IDs to (SourceDefinition, ConfigType) tuples
    """
    return dict(SOURCE_REGISTRY)


__all__ = [
    "SOURCE_REGISTRY",
    "get_all_sources",
    "get_source_and_config",
]



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayAdvertisedProducts(AssetSchema):
    """
    The Display Advertised Products report provides insights into the performance of advertised products within
    display ad campaigns. It includes key metrics such as clicks, impressions, purchases, sales, and viewability rates.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    ad_id: int | None = Field(..., description="The ID of the ad")
    add_to_cart: float | None = Field(..., description="The number of add to cart")
    add_to_cart_clicks: float | None = Field(..., description="The number of add to cart clicks")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart")
    add_to_cart_views: float | None = Field(..., description="The number of add to cart views")
    bid_optimization: str | None = Field(..., description="The bid optimization")
    branded_search_rate: float | None = Field(..., description="The rate of branded searches")
    branded_searches: float | None = Field(..., description="The number of branded searches")
    branded_searches_clicks: float | None = Field(..., description="The number of branded searches clicks")
    branded_searches_views: float | None = Field(..., description="The number of branded searches views")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    clicks: float | None = Field(..., description="The number of clicks")
    cost: float | None = Field(..., description="The cost")
    cumulative_reach: float | None = Field(..., description="The cumulative reach")
    date: datetime.date | None = Field(..., description="The date of the advertisement")
    detail_page_views: float | None = Field(..., description="The number of detail page views")
    detail_page_views_clicks: float | None = Field(..., description="The number of detail page views clicks")
    ecp_add_to_cart: float | None = Field(..., description="The ECP of add to cart")
    ecp_brand_search: float | None = Field(..., description="The ECP of brand search")
    impressions: float | None = Field(..., description="The number of impressions")
    impressions_frequency_average: float | None = Field(..., description="The average frequency of impressions")
    impressions_views: float | None = Field(..., description="The number of impressions views")
    new_to_brand_detail_page_view_clicks: float | None = Field(
        ..., description="The number of new-to-brand detail page view clicks"
    )
    new_to_brand_detail_page_view_rate: float | None = Field(
        ..., description="The rate of new-to-brand detail page views"
    )
    new_to_brand_detail_page_view_views: float | None = Field(
        ..., description="The number of new-to-brand detail page view views"
    )
    new_to_brand_detail_page_views: float | None = Field(
        ..., description="The number of new-to-brand detail page views"
    )
    new_to_brand_ecp_detail_page_view: float | None = Field(
        ..., description="The ECP of new-to-brand detail page views"
    )
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases")
    new_to_brand_purchases_clicks: float | None = Field(..., description="The number of new-to-brand purchases clicks")
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of new-to-brand sales clicks")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new-to-brand units sold")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of new-to-brand units sold clicks"
    )
    promoted_asin: str | None = Field(..., description="The ASIN of the promoted product")
    promoted_sku: str | None = Field(..., description="The SKU of the promoted product")
    purchases: float | None = Field(..., description="The number of purchases")
    purchases_clicks: float | None = Field(..., description="The number of purchases clicks")
    purchases_promoted_clicks: float | None = Field(..., description="The number of purchases promoted clicks")
    sales: float | None = Field(..., description="The number of sales")
    sales_clicks: float | None = Field(..., description="The number of sales clicks")
    sales_promoted_clicks: float | None = Field(..., description="The number of sales promoted clicks")
    units_sold: float | None = Field(..., description="The number of units sold")
    units_sold_clicks: float | None = Field(..., description="The number of units sold clicks")
    video_complete_views: float | None = Field(..., description="The number of video complete views")
    video_first_quartile_views: float | None = Field(..., description="The number of video first quartile views")
    video_midpoint_views: float | None = Field(..., description="The number of video midpoint views")
    video_third_quartile_views: float | None = Field(..., description="The number of video third quartile views")
    video_unmutes: float | None = Field(..., description="The number of video unmutes")
    view_click_through_rate: float | None = Field(..., description="The click-through rate of views")
    viewability_rate: float | None = Field(..., description="The rate of viewability")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class FacebookAds:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Performance insights for advertising campaigns including clicks, impressions, spend, and engagement rates."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Performance and effectiveness of Facebook advertising campaigns including clicks, impressions, spend, and
        engagement actions.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads_by_age_gender(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance by age and gender demographics including clicks, impressions, spend, and engagement rates."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads_by_country(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance across different countries and regions including clicks, impressions, spend, and engagement
        rates.
        """

        raise NotImplementedError



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class Teads:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance including billable events, delivered budget, CPC, CPM, and viewability."""

        raise NotImplementedError



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class Awin:
    @il.asset(tags=["Report"])
    def advertiser(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertiser performance within the Awin network including clicks, impressions, conversions, and revenue."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def advertiser_by_publishers(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Advertiser performance across publishers within the Awin network including clicks, impressions,
        confirmations, and revenue.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def advertiser_transactions(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Transactional activities including clicks, commissions, sales, transaction value, commission rates, and
        conversion data.
        """

        raise NotImplementedError



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class ProductsPurchasedProducts(AssetSchema):
    """
    The Products Purchased Products report provides information about the performance of advertised products.
    It includes key metrics such as sales, purchases, SKU, and units sold within different time frames.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    advertised_asin: str | None = Field(..., description="The ASIN of the advertised product")
    advertised_sku: str | None = Field(..., description="The SKU of the advertised product")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    date: datetime.date | None = Field(..., description="The date of the purchase")
    keyword: str | None = Field(..., description="The keyword")
    keyword_id: int | None = Field(..., description="The ID of the keyword")
    keyword_type: str | None = Field(..., description="The type of keyword")
    kindle_edition_normalized_pages_read_14d: float | None = Field(
        ..., description="The number of Kindle Edition Normalized Pages read in the last 14 days"
    )
    kindle_edition_normalized_pages_royalties_14d: float | None = Field(
        ..., description="The royalties for Kindle Edition Normalized Pages in the last 14 days"
    )
    match_type: str | None = Field(..., description="The type of match")
    portfolio_id: int | None = Field(..., description="The ID of the portfolio")
    purchased_asin: str | None = Field(..., description="The ASIN of the purchased product")
    purchases_14d: float | None = Field(..., description="The number of purchases in the last 14 days")
    purchases_1d: float | None = Field(..., description="The number of purchases in the last 1 day")
    purchases_30d: float | None = Field(..., description="The number of purchases in the last 30 days")
    purchases_7d: float | None = Field(..., description="The number of purchases in the last 7 days")
    purchases_other_sku_14d: float | None = Field(
        ..., description="The number of purchases for other SKUs in the last 14 days"
    )
    purchases_other_sku_1d: float | None = Field(
        ..., description="The number of purchases for other SKUs in the last 1 day"
    )
    purchases_other_sku_30d: float | None = Field(
        ..., description="The number of purchases for other SKUs in the last 30 days"
    )
    purchases_other_sku_7d: float | None = Field(
        ..., description="The number of purchases for other SKUs in the last 7 days"
    )
    sales_14d: float | None = Field(..., description="The sales in the last 14 days")
    sales_1d: float | None = Field(..., description="The sales in the last 1 day")
    sales_30d: float | None = Field(..., description="The sales in the last 30 days")
    sales_7d: float | None = Field(..., description="The sales in the last 7 days")
    sales_other_sku_14d: float | None = Field(..., description="The sales for other SKUs in the last 14 days")
    sales_other_sku_1d: float | None = Field(..., description="The sales for other SKUs in the last 1 day")
    sales_other_sku_30d: float | None = Field(..., description="The sales for other SKUs in the last 30 days")
    sales_other_sku_7d: float | None = Field(..., description="The sales for other SKUs in the last 7 days")
    units_sold_clicks_14d: float | None = Field(
        ..., description="The number of units sold from clicks in the last 14 days"
    )
    units_sold_clicks_1d: float | None = Field(
        ..., description="The number of units sold from clicks in the last 1 day"
    )
    units_sold_clicks_30d: float | None = Field(
        ..., description="The number of units sold from clicks in the last 30 days"
    )
    units_sold_clicks_7d: float | None = Field(
        ..., description="The number of units sold from clicks in the last 7 days"
    )
    units_sold_other_sku_14d: float | None = Field(
        ..., description="The number of units sold for other SKUs in the last 14 days"
    )
    units_sold_other_sku_1d: float | None = Field(
        ..., description="The number of units sold for other SKUs in the last 1 day"
    )
    units_sold_other_sku_30d: float | None = Field(
        ..., description="The number of units sold for other SKUs in the last 30 days"
    )
    units_sold_other_sku_7d: float | None = Field(
        ..., description="The number of units sold for other SKUs in the last 7 days"
    )



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class BrandsPurchasedProducts(AssetSchema):
    """
    The Brands Purchased Products report provides insights into the performance of specific products purchased through
    Amazon Ads campaigns. It includes key metrics such as the number of purchases, sales, and units sold in the last
    X days, along with details about new-to-brand purchases and sales.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group")
    ad_group_name: str | None = Field(..., description="The name of the ad group")
    attribution_type: str | None = Field(..., description="The type of attribution")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget")
    campaign_id: int | None = Field(..., description="The ID of the campaign")
    campaign_name: str | None = Field(..., description="The name of the campaign")
    date: datetime.date | None = Field(..., description="The date of the record")
    new_to_brand_purchases_14d: float | None = Field(
        ..., description="The number of purchases made by new customers in the last 14 days"
    )
    new_to_brand_purchases_percentage_14d: float | None = Field(
        ..., description="The percentage of purchases made by new customers in the last 14 days"
    )
    new_to_brand_sales_14d: float | None = Field(
        ..., description="The total sales made by new customers in the last 14 days"
    )
    new_to_brand_sales_percentage_14d: float | None = Field(
        ..., description="The percentage of sales made by new customers in the last 14 days"
    )
    new_to_brand_units_sold_14d: float | None = Field(
        ..., description="The number of units sold to new customers in the last 14 days"
    )
    new_to_brand_units_sold_percentage_14d: float | None = Field(
        ..., description="The percentage of units sold to new customers in the last 14 days"
    )
    orders_14d: float | None = Field(..., description="The total number of orders in the last 14 days")
    product_category: str | None = Field(..., description="The category of the product")
    product_name: str | None = Field(..., description="The name of the product")
    purchased_asin: str | None = Field(..., description="The ASIN of the purchased product")
    sales_14d: float | None = Field(..., description="The total sales in the last 14 days")
    units_sold_14d: float | None = Field(..., description="The total number of units sold in the last 14 days")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class SnapchatAds:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance including attachment quartiles, view time, conversion rates, impressions, and spend."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Individual ad performance including attachment views, conversions, custom events, impressions, saves, and
        spend.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads_by_country(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance by country including view time, conversion events, impressions, and spend."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads_geo(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance by geographical data including attachment views, conversions, impressions, and spend."""

        raise NotImplementedError



from interloper.schema import AssetSchema
from pydantic import Field


class ConversionsReport(AssetSchema):
    """
    The Conversions report provides insights on conversion events tracked within campaigns.
    It includes key metrics such as the amount of conversions, conversion time, currency, IP geolocation data,
    media information, order details, and user device information such as browser, device, and operating system.
    """

    agent_id: int = Field(description="Agent ID")
    amount: str = Field(description="Amount")
    camp_id: int = Field(description="Campaign ID")
    camp_title: str = Field(description="Campaign Title")
    cart_id: int = Field(description="Cart ID")
    click_ref_id: int = Field(description="Click Reference ID")
    click_stamp: int = Field(description="Click Stamp")
    coid: int = Field(description="COID")
    company_name: str = Field(description="Company Name")
    conversion_time: int = Field(description="Conversion Time")
    currency: str = Field(description="Currency")
    id: int = Field(description="ID")
    ip_city: str = Field(description="IP City")
    ip_country_geoname_id: int = Field(description="IP Country Geoname ID")
    ip_country_iso_code: str = Field(description="IP Country ISO Code")
    ip_country_names_de: str = Field(description="IP Country Names (German)")
    ip_country_names_en: str = Field(description="IP Country Names (English)")
    ip_country_names_es: str = Field(description="IP Country Names (Spanish)")
    ip_country_names_fr: str = Field(description="IP Country Names (French)")
    ip_country_names_ja: str = Field(description="IP Country Names (Japanese)")
    ip_country_names_pt_br: str = Field(description="IP Country Names (Portuguese - Brazil)")
    ip_country_names_ru: str = Field(description="IP Country Names (Russian)")
    ip_country_names_zh_cn: str = Field(description="IP Country Names (Chinese - Simplified)")
    leaddata: str = Field(description="Lead Data")
    media_id: int = Field(description="Media ID")
    media_url: int = Field(description="Media URL")
    medianame: str = Field(description="Media Name")
    order_id: str = Field(description="Order ID")
    price: str = Field(description="Price")
    pricevariable: str = Field(description="Price Variable")
    reject_reason: int = Field(description="Reject Reason")
    result: str = Field(description="Result")
    sensitive_data: int = Field(description="Sensitive Data")
    stamp: str = Field(description="Stamp")
    status: str = Field(description="Status")
    user_agent: str = Field(description="User Agent")
    user_browser: int = Field(description="User Browser")
    user_device: int = Field(description="User Device")
    user_os: int = Field(description="User OS")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class PinterestAds:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance on Pinterest including conversions, click-through rates, engagement rates, costs per
        click, impressions, and spend.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance on Pinterest including click-through conversions, engagement rates, costs per click,
        impressions, and spend.
        """

        raise NotImplementedError



from .brands_ad_groups import BrandsAdGroups
from .brands_ads import BrandsAds
from .brands_campaigns import BrandsCampaigns
from .brands_gross_and_invalid_traffic import BrandsGrossAndInvalidTraffic
from .brands_placements import BrandsPlacements
from .brands_purchased_products import BrandsPurchasedProducts
from .brands_search_terms import BrandsSearchTerms
from .brands_targeting import BrandsTargeting
from .display_ad_groups import DisplayAdGroups
from .display_advertised_products import DisplayAdvertisedProducts
from .display_campaigns import DisplayCampaigns
from .display_gross_and_invalid_traffic import DisplayGrossAndInvalidTraffic
from .display_purchased_products import DisplayPurchasedProducts
from .display_targeting import DisplayTargeting
from .products_advertised_products import ProductsAdvertisedProducts
from .products_campaigns import ProductsCampaigns
from .products_gross_and_invalid_traffic import ProductsGrossAndInvalidTraffic
from .products_purchased_products import ProductsPurchasedProducts
from .products_search_terms import ProductsSearchTerms
from .products_targeting import ProductsTargeting
from .profiles import Profiles

__all__ = [
    "BrandsAdGroups",
    "BrandsAds",
    "BrandsCampaigns",
    "BrandsGrossAndInvalidTraffic",
    "BrandsPlacements",
    "BrandsPurchasedProducts",
    "BrandsSearchTerms",
    "BrandsTargeting",
    "DisplayAdGroups",
    "DisplayAdvertisedProducts",
    "DisplayCampaigns",
    "DisplayGrossAndInvalidTraffic",
    "DisplayPurchasedProducts",
    "DisplayTargeting",
    "ProductsAdvertisedProducts",
    "ProductsCampaigns",
    "ProductsGrossAndInvalidTraffic",
    "ProductsPurchasedProducts",
    "ProductsSearchTerms",
    "ProductsTargeting",
    "Profiles",
]



import datetime

from interloper.schema import AssetSchema
from pydantic import Field


class DisplayAdGroups(AssetSchema):
    """
    The Display Ad Groups report provides insights into the performance of ad groups within display advertising
    campaigns. It includes key metrics such as clicks, impressions, purchases, sales, and viewability rates.
    """

    ad_group_id: int | None = Field(..., description="The ID of the ad group.")
    ad_group_name: str | None = Field(..., description="The name of the ad group.")
    add_to_cart: float | None = Field(..., description="The number of add to cart.")
    add_to_cart_clicks: float | None = Field(..., description="The number of add to cart clicks.")
    add_to_cart_rate: float | None = Field(..., description="The rate of add to cart.")
    add_to_cart_views: float | None = Field(..., description="The number of add to cart views.")
    bid_optimization: str | None = Field(..., description="The bid optimization strategy.")
    branded_search_rate: float | None = Field(..., description="The rate of branded searches.")
    branded_searches: float | None = Field(..., description="The number of branded searches.")
    branded_searches_clicks: float | None = Field(..., description="The number of branded searches clicks.")
    branded_searches_views: float | None = Field(..., description="The number of branded searches views.")
    campaign_budget_currency_code: str | None = Field(..., description="The currency code of the campaign budget.")
    campaign_id: int | None = Field(..., description="The ID of the campaign.")
    campaign_name: str | None = Field(..., description="The name of the campaign.")
    clicks: float | None = Field(..., description="The number of clicks.")
    cost: float | None = Field(..., description="The cost of the ad group.")
    date: datetime.date | None = Field(..., description="The date of the ad group.")
    detail_page_views: float | None = Field(..., description="The number of detail page views.")
    detail_page_views_clicks: float | None = Field(..., description="The number of detail page views clicks.")
    ecp_add_to_cart: float | None = Field(..., description="The effective cost per add to cart.")
    ecp_brand_search: float | None = Field(..., description="The effective cost per brand search.")
    impressions: float | None = Field(..., description="The number of impressions.")
    impressions_views: float | None = Field(..., description="The number of impressions views.")
    new_to_brand_purchases: float | None = Field(..., description="The number of new-to-brand purchases.")
    new_to_brand_purchases_clicks: float | None = Field(..., description="The number of new-to-brand purchases clicks.")
    new_to_brand_sales: float | None = Field(..., description="The number of new-to-brand sales.")
    new_to_brand_sales_clicks: float | None = Field(..., description="The number of new-to-brand sales clicks.")
    new_to_brand_units_sold: float | None = Field(..., description="The number of new-to-brand units sold.")
    new_to_brand_units_sold_clicks: float | None = Field(
        ..., description="The number of new-to-brand units sold clicks."
    )
    purchases: float | None = Field(..., description="The number of purchases.")
    purchases_clicks: float | None = Field(..., description="The number of purchases clicks.")
    purchases_promoted_clicks: float | None = Field(..., description="The number of purchases promoted clicks.")
    sales: float | None = Field(..., description="The number of sales.")
    sales_clicks: float | None = Field(..., description="The number of sales clicks.")
    sales_promoted_clicks: float | None = Field(..., description="The number of sales promoted clicks.")
    units_sold: float | None = Field(..., description="The number of units sold.")
    units_sold_clicks: float | None = Field(..., description="The number of units sold clicks.")
    video_complete_views: float | None = Field(..., description="The number of video complete views.")
    video_first_quartile_views: float | None = Field(..., description="The number of video first quartile views.")
    video_midpoint_views: float | None = Field(..., description="The number of video midpoint views.")
    video_third_quartile_views: float | None = Field(..., description="The number of video third quartile views.")
    video_unmutes: float | None = Field(..., description="The number of video unmutes.")
    view_click_through_rate: float | None = Field(..., description="The rate of view click through.")
    viewability_rate: float | None = Field(..., description="The rate of viewability.")



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class CampaignManager360:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign performance and effectiveness including impressions, clicks, CTR, conversions, and cost."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Ad performance within Campaign Manager 360 including impressions, clicks, conversions, and cost."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def reach(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Reach and engagement metrics focusing on unique user impressions and reach frequency."""

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def custom_audiences(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Audience data for Campaign Manager 360."""

        raise NotImplementedError



# ------------------------------------------------------------------
# PRODUCTS
# ------------------------------------------------------------------

PRODUCTS_CAMPAIGN_METRICS = [
    "adGroupId",
    "adGroupName",
    "adStatus",
    "attributedSalesSameSku14d",
    "attributedSalesSameSku1d",
    "attributedSalesSameSku30d",
    "attributedSalesSameSku7d",
    "campaignApplicableBudgetRuleId",
    "campaignApplicableBudgetRuleName",
    "campaignBiddingStrategy",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignRuleBasedBudgetAmount",
    "campaignStatus",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerClick",
    "date",
    "impressions",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "purchases14d",
    "purchases1d",
    "purchases30d",
    "purchases7d",
    "purchasesSameSku14d",
    "purchasesSameSku1d",
    "purchasesSameSku30d",
    "purchasesSameSku7d",
    "sales14d",
    "sales1d",
    "sales30d",
    "sales7d",
    "spend",
    "unitsSoldClicks14d",
    "unitsSoldClicks1d",
    "unitsSoldClicks30d",
    "unitsSoldClicks7d",
    "unitsSoldSameSku14d",
    "unitsSoldSameSku1d",
    "unitsSoldSameSku30d",
    "unitsSoldSameSku7d",
]

PRODUCTS_TARGETING_METRICS = [
    "acosClicks14d",
    "acosClicks7d",
    "adGroupId",
    "adGroupName",
    "attributedSalesSameSku14d",
    "attributedSalesSameSku1d",
    "attributedSalesSameSku30d",
    "attributedSalesSameSku7d",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerClick",
    "date",
    "impressions",
    "keyword",
    "keywordBid",
    "keywordId",
    "keywordType",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "matchType",
    "portfolioId",
    "purchases14d",
    "purchases1d",
    "purchases30d",
    "purchases7d",
    "purchasesSameSku14d",
    "purchasesSameSku1d",
    "purchasesSameSku30d",
    "purchasesSameSku7d",
    "roasClicks14d",
    "roasClicks7d",
    "sales14d",
    "sales1d",
    "sales30d",
    "sales7d",
    "salesOtherSku7d",
    "targeting",
    "topOfSearchImpressionShare",
    "unitsSoldClicks14d",
    "unitsSoldClicks1d",
    "unitsSoldClicks30d",
    "unitsSoldClicks7d",
    "unitsSoldOtherSku7d",
    "unitsSoldSameSku14d",
    "unitsSoldSameSku1d",
    "unitsSoldSameSku30d",
    "unitsSoldSameSku7d",
]

PRODUCTS_SEARCH_TERM_METRICS = [
    "acosClicks14d",
    "acosClicks7d",
    "adGroupId",
    "adGroupName",
    "adKeywordStatus",
    "attributedSalesSameSku14d",
    "attributedSalesSameSku1d",
    "attributedSalesSameSku30d",
    "attributedSalesSameSku7d",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerClick",
    "date",
    "impressions",
    "keyword",
    "keywordBid",
    "keywordId",
    "keywordType",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "matchType",
    "portfolioId",
    "purchases14d",
    "purchases1d",
    "purchases30d",
    "purchases7d",
    "purchasesSameSku14d",
    "purchasesSameSku1d",
    "purchasesSameSku30d",
    "purchasesSameSku7d",
    "roasClicks14d",
    "roasClicks7d",
    "sales14d",
    "sales1d",
    "sales30d",
    "sales7d",
    "salesOtherSku7d",
    "searchTerm",
    "targeting",
    "unitsSoldClicks14d",
    "unitsSoldClicks1d",
    "unitsSoldClicks30d",
    "unitsSoldClicks7d",
    "unitsSoldOtherSku7d",
    "unitsSoldSameSku14d",
    "unitsSoldSameSku1d",
    "unitsSoldSameSku30d",
    "unitsSoldSameSku7d",
]

PRODUCTS_ADVERTISED_PRODUCT_METRICS = [
    "acosClicks14d",
    "acosClicks7d",
    "adGroupId",
    "adGroupName",
    "adId",
    "advertisedAsin",
    "advertisedSku",
    "attributedSalesSameSku14d",
    "attributedSalesSameSku1d",
    "attributedSalesSameSku30d",
    "attributedSalesSameSku7d",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerClick",
    "date",
    "impressions",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "portfolioId",
    "purchases14d",
    "purchases1d",
    "purchases30d",
    "purchases7d",
    "purchasesSameSku14d",
    "purchasesSameSku1d",
    "purchasesSameSku30d",
    "purchasesSameSku7d",
    "roasClicks14d",
    "roasClicks7d",
    "sales14d",
    "sales1d",
    "sales30d",
    "sales7d",
    "salesOtherSku7d",
    "spend",
    "unitsSoldClicks14d",
    "unitsSoldClicks1d",
    "unitsSoldClicks30d",
    "unitsSoldClicks7d",
    "unitsSoldOtherSku7d",
    "unitsSoldSameSku14d",
    "unitsSoldSameSku1d",
    "unitsSoldSameSku30d",
    "unitsSoldSameSku7d",
]

PRODUCTS_PURCHASED_PRODUCT_METRICS = [
    "adGroupId",
    "adGroupName",
    "advertisedAsin",
    "advertisedSku",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "date",
    "keyword",
    "keywordId",
    "keywordType",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "matchType",
    "portfolioId",
    "purchasedAsin",
    "purchases14d",
    "purchases1d",
    "purchases30d",
    "purchases7d",
    "purchasesOtherSku14d",
    "purchasesOtherSku1d",
    "purchasesOtherSku30d",
    "purchasesOtherSku7d",
    "sales14d",
    "sales1d",
    "sales30d",
    "sales7d",
    "salesOtherSku14d",
    "salesOtherSku1d",
    "salesOtherSku30d",
    "salesOtherSku7d",
    "unitsSoldClicks14d",
    "unitsSoldClicks1d",
    "unitsSoldClicks30d",
    "unitsSoldClicks7d",
    "unitsSoldOtherSku14d",
    "unitsSoldOtherSku1d",
    "unitsSoldOtherSku30d",
    "unitsSoldOtherSku7d",
]

PRODUCTS_GROSS_AND_INVALID_TRAFFIC_METRICS = [
    "campaignName",
    "campaignStatus",
    "clicks",
    "date",
    "grossClickThroughs",
    "grossImpressions",
    "impressions",
    "invalidClickThroughRate",
    "invalidClickThroughs",
    "invalidImpressionRate",
    "invalidImpressions",
]

# ------------------------------------------------------------------
# BRANDS
# ------------------------------------------------------------------

BRANDS_CAMPAIGN_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "brandedSearches",
    "brandedSearchesClicks",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignRuleBasedBudgetAmount",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "impressions",
    "kindleEditionNormalizedPagesRead14d",
    "kindleEditionNormalizedPagesRoyalties14d",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewsClicks",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesPercentage",
    "newToBrandPurchasesRate",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesPercentage",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "newToBrandUnitsSoldPercentage",
    "purchases",
    "purchasesClicks",
    "purchasesPromoted",
    "sales",
    "salesClicks",
    "salesPromoted",
    "topOfSearchImpressionShare",
    "unitsSold",
    "unitsSoldClicks",
    "video5SecondViewRate",
    "video5SecondViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewableImpressions",
    "viewClickThroughRate",
]

BRANDS_AD_GROUP_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "adGroupId",
    "adGroupName",
    "adStatus",
    "brandedSearches",
    "brandedSearchesClicks",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "impressions",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewsClicks",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesPercentage",
    "newToBrandPurchasesRate",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesPercentage",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "newToBrandUnitsSoldPercentage",
    "purchases",
    "purchasesClicks",
    "purchasesPromoted",
    "sales",
    "salesClicks",
    "salesPromoted",
    "unitsSold",
    "unitsSoldClicks",
    "video5SecondViewRate",
    "video5SecondViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
]

BRANDS_PLACEMENT_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "brandedSearches",
    "brandedSearchesClicks",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "impressions",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewsClicks",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesPercentage",
    "newToBrandPurchasesRate",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesPercentage",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "newToBrandUnitsSoldPercentage",
    "purchases",
    "purchasesClicks",
    "purchasesPromoted",
    "sales",
    "salesClicks",
    "salesPromoted",
    "unitsSold",
    "unitsSoldClicks",
    "video5SecondViewRate",
    "video5SecondViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewableImpressions",
    "viewClickThroughRate",
]

BRANDS_TARGETING_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "adGroupId",
    "adGroupName",
    "adKeywordStatus",
    "brandedSearches",
    "brandedSearchesClicks",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "impressions",
    "keywordBid",
    "keywordId",
    "keywordText",
    "keywordType",
    "matchType",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewsClicks",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesPercentage",
    "newToBrandPurchasesRate",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesPercentage",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "newToBrandUnitsSoldPercentage",
    "purchases",
    "purchasesClicks",
    "purchasesPromoted",
    "sales",
    "salesClicks",
    "salesPromoted",
    "targetingExpression",
    "targetingId",
    "targetingText",
    "targetingType",
    "topOfSearchImpressionShare",
]

BRANDS_SEARCH_TERM_METRICS = [
    "adGroupId",
    "adGroupName",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "impressions",
    "keywordBid",
    "keywordId",
    "keywordText",
    "matchType",
    "purchases",
    "purchasesClicks",
    "sales",
    "salesClicks",
    "searchTerm",
    "unitsSold",
    "video5SecondViewRate",
    "video5SecondViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewableImpressions",
    "viewClickThroughRate",
]

BRANDS_AD_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "adGroupId",
    "adGroupName",
    "adId",
    "brandedSearches",
    "brandedSearchesClicks",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "impressions",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewsClicks",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesPercentage",
    "newToBrandPurchasesRate",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesPercentage",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "newToBrandUnitsSoldPercentage",
    "purchases",
    "purchasesClicks",
    "purchasesPromoted",
    "sales",
    "salesClicks",
    "salesPromoted",
    "unitsSold",
    "unitsSoldClicks",
    "video5SecondViewRate",
    "video5SecondViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewableImpressions",
]

BRANDS_PURCHASED_PRODUCT_METRICS = [
    "adGroupId",
    "adGroupName",
    "attributionType",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "date",
    "newToBrandPurchases14d",
    "newToBrandPurchasesPercentage14d",
    "newToBrandSales14d",
    "newToBrandSalesPercentage14d",
    "newToBrandUnitsSold14d",
    "newToBrandUnitsSoldPercentage14d",
    "orders14d",
    "productCategory",
    "productName",
    "purchasedAsin",
    "sales14d",
    "unitsSold14d",
]

BRANDS_GROSS_AND_INVALID_TRAFFIC_METRICS = [
    "campaignName",
    "campaignStatus",
    "clicks",
    "date",
    "grossClickThroughs",
    "grossImpressions",
    "impressions",
    "invalidClickThroughRate",
    "invalidClickThroughs",
    "invalidImpressionRate",
    "invalidImpressions",
]

# ------------------------------------------------------------------
# DISPLAY
# ------------------------------------------------------------------

DISPLAY_CAMPAIGN_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "addToCartViews",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "brandedSearchRate",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "cost",
    "costType",
    "cumulativeReach",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "eCPBrandSearch",
    "impressions",
    "impressionsFrequencyAverage",
    "impressionsViews",
    "newToBrandDetailPageViewClicks",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewViews",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "purchases",
    "purchasesClicks",
    "purchasesPromotedClicks",
    "sales",
    "salesClicks",
    "salesPromotedClicks",
    "unitsSold",
    "unitsSoldClicks",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewClickThroughRate",
]

DISPLAY_AD_GROUP_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "addToCartViews",
    "adGroupId",
    "adGroupName",
    "bidOptimization",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "brandedSearchRate",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "clicks",
    "cost",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "eCPBrandSearch",
    "impressions",
    "impressionsViews",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "purchases",
    "purchasesClicks",
    "purchasesPromotedClicks",
    "sales",
    "salesClicks",
    "salesPromotedClicks",
    "unitsSold",
    "unitsSoldClicks",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewClickThroughRate",
]

DISPLAY_TARGETING_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "addToCartViews",
    "adGroupId",
    "adGroupName",
    "adKeywordStatus",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "brandedSearchRate",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "clicks",
    "cost",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "eCPBrandSearch",
    "impressions",
    "impressionsViews",
    "newToBrandDetailPageViewClicks",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewViews",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "purchases",
    "purchasesClicks",
    "purchasesPromotedClicks",
    "sales",
    "salesClicks",
    "salesPromotedClicks",
    "targetingExpression",
    "targetingId",
    "targetingText",
    "unitsSold",
    "unitsSoldClicks",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewClickThroughRate",
]

DISPLAY_ADVERTISED_PRODUCT_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartRate",
    "addToCartViews",
    "adGroupId",
    "adGroupName",
    "adId",
    "bidOptimization",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "brandedSearchRate",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "clicks",
    "cost",
    "cumulativeReach",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "eCPAddToCart",
    "eCPBrandSearch",
    "impressions",
    "impressionsFrequencyAverage",
    "impressionsViews",
    "newToBrandDetailPageViewClicks",
    "newToBrandDetailPageViewRate",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewViews",
    "newToBrandECPDetailPageView",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandUnitsSold",
    "newToBrandUnitsSoldClicks",
    "promotedAsin",
    "promotedSku",
    "purchases",
    "purchasesClicks",
    "purchasesPromotedClicks",
    "sales",
    "salesClicks",
    "salesPromotedClicks",
    "unitsSold",
    "unitsSoldClicks",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
    "videoUnmutes",
    "viewabilityRate",
    "viewClickThroughRate",
]

DISPLAY_PURCHASED_PRODUCT_METRICS = [
    "adGroupId",
    "adGroupName",
    "asinBrandHalo",
    "campaignBudgetCurrencyCode",
    "campaignId",
    "campaignName",
    "conversionsBrandHalo",
    "conversionsBrandHaloClicks",
    "date",
    "promotedAsin",
    "promotedSku",
    "salesBrandHalo",
    "salesBrandHaloClicks",
    "unitsSoldBrandHalo",
    "unitsSoldBrandHaloClicks",
]

DISPLAY_GROSS_AND_INVALID_TRAFFIC_METRICS = [
    "campaignName",
    "campaignStatus",
    "clicks",
    "date",
    "grossClickThroughs",
    "grossImpressions",
    "impressions",
    "invalidClickThroughRate",
    "invalidClickThroughs",
    "invalidImpressionRate",
    "invalidImpressions",
]

# ------------------------------------------------------------------
# TELEVISION
# ------------------------------------------------------------------

TELEVISION_CAMPAIGN_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartViews",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "campaignBudgetAmount",
    "campaignBudgetCurrencyCode",
    "campaignBudgetType",
    "campaignId",
    "campaignName",
    "campaignStatus",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerThousandImpressions",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "detailPageViewsViews",
    "impressions",
    "newToBrandDetailPageViewClicks",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewViews",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesViews",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesViews",
    "portfolioId",
    "purchases",
    "purchasesClicks",
    "purchasesViews",
    "roas",
    "sales",
    "salesClicks",
    "salesViews",
    "unitsSold",
    "unitsSoldClicks",
    "unitsSoldViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
]

TELEVISION_TARGETING_METRICS = [
    "addToCart",
    "addToCartClicks",
    "addToCartViews",
    "brandedSearches",
    "brandedSearchesClicks",
    "brandedSearchesViews",
    "clicks",
    "clickThroughRate",
    "cost",
    "costPerThousandImpressions",
    "date",
    "detailPageViews",
    "detailPageViewsClicks",
    "detailPageViewsViews",
    "impressions",
    "keyword",
    "keywordBid",
    "keywordId",
    "keywordType",
    "matchType",
    "newToBrandDetailPageViewClicks",
    "newToBrandDetailPageViews",
    "newToBrandDetailPageViewViews",
    "newToBrandPurchases",
    "newToBrandPurchasesClicks",
    "newToBrandPurchasesViews",
    "newToBrandSales",
    "newToBrandSalesClicks",
    "newToBrandSalesViews",
    "portfolioId",
    "purchases",
    "purchasesClicks",
    "purchasesViews",
    "roas",
    "sales",
    "salesClicks",
    "salesViews",
    "targetingText",
    "unitsSold",
    "unitsSoldClicks",
    "unitsSoldViews",
    "videoCompleteViews",
    "videoFirstQuartileViews",
    "videoMidpointViews",
    "videoThirdQuartileViews",
]



import interloper as il
import pandas as pd


@il.source(tags=["Advertising"])
class Criteo:
    @il.asset(tags=["Report"])
    def campaigns(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Campaign effectiveness across multiple channels and attribution models including conversion rates and return
        on ad spend.
        """

        raise NotImplementedError

    @il.asset(tags=["Report"])
    def ads(self, context: il.ExecutionContext) -> pd.DataFrame:
        """Marketing campaign performance including click-through rates, conversion rates, cost per acquisition, and
        return on ad spend.
        """

        raise NotImplementedError



from interloper.schema import AssetSchema
from pydantic import Field


class CampaignsByDeviceType(AssetSchema):
    """
    The Campaigns by Device Type report provides insights into campaign performance
    based on the type of device used by users. It includes key metrics such as
    conversion rates, the number of conversions, and user demographics such as
    browser usage, operating system, city, and country.
    """

    agent_id: int = Field(description="The ID of the agent")
    browser: str = Field(description="The browser used")
    camp_id: int = Field(description="The ID of the campaign")
    camp_title: str = Field(description="The title of the campaign")
    city: str = Field(description="The city where the campaign was run")
    conversion_pct: int = Field(description="The percentage of conversions")
    conversions: int = Field(description="The number of conversions")
    country: str = Field(description="The country where the campaign was run")
    device_model: str = Field(description="The model of the device")
    device_type: str = Field(description="The type of the device")
    os: str = Field(description="The operating system used")
